<?php
namespace YpmPopup;
use \WP_Query;
use ypmFrontend\ConditionsChecker;
use ypmFrontend\DisplayChecker;

abstract class Popup {

	public $shortCodeName = 'ypm_popup';

	private $sanitizedData;
	private $popupId;
	private $content;
	private $options = array();

	public function __construct()
	{

	}

	public function setPopupId($popupId) {

		$this->popupId = $popupId;
	}

	public function getId()
	{
		return $this->popupId;
	}

	public function getPopupId() {

		return $this->popupId;
	}

	public function setOptions()
	{
		$savedData = Popup::getSavedData($this->getId());
		$this->options = $savedData;
	}

	public function getOptions()
	{
		return $this->options;
	}

	public function setContent($content)
	{
		$this->content = $content;
	}

	public function getContent()
	{
		return $this->content;
	}

	public function insertIntoSanitizedData($sanitizedData) {

		if(!empty($sanitizedData)) {
			$this->sanitizedData[$sanitizedData['name']] = $sanitizedData['value'];
		}
	}

	public function setSanitizedData($sanitizedData) {

		$this->sanitizedData = $sanitizedData;
	}

	public function getSanitizedData() {

		return $this->sanitizedData;
	}

	public function sanitizeValueByType($value, $type) {

		$sanitizedValue = '';
		switch ($type) {
			case 'string':
				$sanitizedValue = sanitize_text_field($value);
				break;
			case 'array':
				$sanitizedValue = $this->recursiveSanitizeTextField($value);
				break;
			case 'email':
				$sanitizedValue = sanitize_email($value);
				break;
			case 'textMessage':
				$sanitizedValue = $value;
				break;
			default:
				$sanitizedValue = sanitize_text_field($value);
				break;
		}

		return $sanitizedValue;
	}

	public function recursiveSanitizeTextField($array) {

		foreach ($array as $key => &$value) {
			if (is_array($value)) {
				$value = $this->recursiveSanitizeTextField($value);
			}
			else {
				$value = sanitize_text_field($value);
			}
		}

		return $array;
	}

	public static function create($data, $obj) {

		$id = $data['ypm-popup-id'];
		$obj->setPopupId($id);

		foreach($data as $name => $value) {
			if(strpos($name, 'ypm') === 0) {
				$defaultData = $obj->getDefaultDataByName($name);
				if(empty($defaultData['type'])) {
					$defaultData['type'] = 'string';
				}

				$obj->saveDispayConditionsSettings($data);
				$obj->saveConditionConditionsSettings($data);
				$sanitizedValue = $obj->sanitizeValueByType($value, $defaultData['type']);
				$obj->insertIntoSanitizedData(array('name' => $name,'value' => $sanitizedValue));
			}

		}

		$obj->save();
	}

	private function saveDispayConditionsSettings($data) {
		if(empty($data['ypm-display-settings'])) {
			return '';
		}

		$postId = $this->getId();
		$settings = $data['ypm-display-settings'];

		$obj = new DisplayConditionBuilder();
		$obj->setSavedData($settings);
		$settings = $obj->filterForSave();

		update_post_meta($postId, 'ypm-display-settings', $settings);
	}

	private function saveConditionConditionsSettings($data) {
		if(empty($data['ypm-conditions-settings'])) {
			return '';
		}

		$postId = $this->getId();
		$settings = $data['ypm-conditions-settings'];

		$obj = new ConditionsConditionBuilder();
		$obj->setSavedData($settings);
		$settings = $obj->filterForSave();

		update_post_meta($postId, 'ypm-conditions-settings', $settings);
	}

	public function save() {

		$data = $this->getSanitizedData();
		update_post_meta($data['ypm-popup-id'], "ypm-data", $data);
	}

	public function getOptionValue($optionName, $forceDefaultValue = false)
	{
		$savedData = Popup::getSavedData($this->getPopupId());
		$defaultData = $this->getDefaultDataByName($optionName);

        $optionValue = null;

        if (empty($defaultData['type'])) {
            $defaultData['type'] = 'string';
        }

        if (!empty($savedData)) { //edit mode
            if (isset($savedData[$optionName])) { //option exists in the database
                $optionValue = $savedData[$optionName];
            }
            /* if it's a checkbox, it may not exist in the db
             * if we don't care about it's existence, return empty string
             * otherwise, go for it's default value
             */
            else if ($defaultData['type'] == 'checkbox' && !$forceDefaultValue) {
                $optionValue = '';
            }
        }

        if (($optionValue === null && !empty($defaultData['defaultValue'])) || ($defaultData['type'] == 'number' && !isset($optionValue))) {
            $optionValue = $defaultData['defaultValue'];
        }

        if ($defaultData['type'] == 'checkbox') {
            $optionValue = $this->boolToChecked($optionValue);
        }

        if(isset($defaultData['ver']) && $defaultData['ver'] > YPM_POPUP_PKG) {
            if (empty($defaultData['allow'])) {
                return $defaultData['defaultValue'];
            }
            else if (!in_array($optionValue, $defaultData['allow'])) {
                return $defaultData['defaultValue'];
            }
        }

		return $optionValue;
	}

	public static function getSavedData($popupId, $saveMode = true) {

		$savedData = @get_post_meta($popupId, 'ypm-data', $saveMode);
		$displaySettings = self::getDisplaySettings($popupId);
		$conditionsSettings = self::getConditionsSettings($popupId);

		if (!empty($displaySettings)) {
			$savedData['ypm-display-settings'] = $displaySettings;
		}
		if (!empty($conditionsSettings)) {
			$savedData['ypm-conditions-settings'] = $conditionsSettings;
		}

		return apply_filters('ypmSavedData', $savedData);
	}

	public static function getDisplaySettings($postId) {
		$savedData = get_post_meta($postId, 'ypm-display-settings', true);

		return $savedData;
	}

	public static function getConditionsSettings($postId) {
		$savedData = get_post_meta($postId, 'ypm-conditions-settings', true);

		return $savedData;
	}

	private function getDefaultDataByName($optionName) {

		global $YpmDefaults;

		foreach($YpmDefaults as $option) {
			if($option['name'] == $optionName) {
				return $option;
			}
		}

		return array('name' => $optionName, 'type' => 'text', 'defaultValue' => '');
	}

	public static function getPopupIdTitleData() {

		$popupMasterData  = get_post_type_object(YPM_POPUP_POST_TYPE);
		$args = array(
			'orderby'          => 'date',
			'order'            => 'DESC',
			'post_status'      => 'publish',
			'post_type'        =>  YPM_POPUP_POST_TYPE,
		);
		$popupMasterData  = get_posts($args);
		$data = array();

		foreach($popupMasterData as $postData) {

			$data[$postData->ID] = $postData->post_title;
		}

		return $data;
	}

	public function getSiteLocale() {

		$locale = get_bloginfo('language');
		$locale = str_replace('-', '_', $locale);

		return $locale;
	}

	public function boolToChecked($var) {
		return ($var?'checked':'');
	}

	public static function getModulesDataArray() {

		global $YpmPostTypesInfo;
		$modulesDataArray = array();
		$postTypes = $YpmPostTypesInfo['postTypes'];

		if(empty($postTypes)) {
			return $modulesDataArray;
		}
		unset($postTypes[YPM_POPUP_POST_TYPE]);
		foreach($postTypes as $postType => $postTypeLevel) {
			$moduleName = str_replace('ypm', '', $postType);
			$moduleName = ucfirst(strtolower($moduleName));

			$query = new WP_Query(
				array(
					'post_type'      => $postType,
					'posts_per_page' => - 1
				)
			);

			$currentModule = array();
			if($query->have_posts()) {
				/*We check all the popups one by one to realize whether they might be loaded or not.*/
				while($query->have_posts()) {
					$query->next_post();
					$id = $query->post->ID;

					$currentModule[$id] = $query->post->post_title;
				}
				$modulesDataArray[$moduleName] = $currentModule;
			}
			else {
				$modulesDataArray[$moduleName] = array('' => __('No data', YPM_POPUP_TEXT_DOMAIN));
			}
		}
		return $modulesDataArray;
	}

	public static function isPostPublished($postId) {

		$status = true;
		$postStatus = get_post_status($postId);
		
		if(empty($postStatus) || get_post_status($postId) != 'publish') {
			$status = false;
		}

		return $status;
	}

	public static function findByIdAndType($postType, $postId = 0) {

		$ypmPostType = str_replace('ypm','',$postType);
		$ypmPostType = ucfirst(strtolower($ypmPostType));
		$popupClassName = $ypmPostType.'Popup';
		if (!file_exists(YPM_POPUPS .$popupClassName.'.php')) {
			return false;
		}
		require_once(YPM_POPUPS .$popupClassName.'.php');
		$className = __NAMESPACE__.'\\'.$popupClassName;
		$popupTypeObj = new $className;
		$popupTypeObj->setPopupId($postId);
		$popupTypeObj->setOptions();

		return $popupTypeObj;
	}

    public static function find($popupId, $args = array())
    {
        global $YpmPostTypesInfo;
        $popupPostType = get_post_type(sanitize_text_field($popupId));

        $popupPostTypes = $YpmPostTypesInfo['postTypes'];
        if ($popupPostType == YPM_POPUP_POST_TYPE || empty($popupPostTypes[$popupPostType])) {
            $popupPostType = 'ypmHtml';
        }

        $popupTypeObj = Popup::findByIdAndType($popupPostType, $popupId);

        return $popupTypeObj;
    }

    public function allowToLoad()
    {
	    global $post;
	    $popupChecker = DisplayChecker::instance();
	    $loadableModes = $popupChecker->isLoadable($this, $post);

	    return ($loadableModes['attr_event'] || $loadableModes['option_event']);
    }

	public function isActive()
	{
		return $this->getOptionValue('ypm-is-active', true);
	}

	public static function getCustomInsertedDataByPostId()
	{
		return array();
	}
}